@@ -10,7 +10,7 @@ use crate::{
     Element, PyArray,
 };
 
-/// Covnersion trait from some rust types to `PyArray`.
+/// Conversion trait from some rust types to `PyArray`.
 ///
 /// This trait takes `self`, which means **it holds a pointer to Rust heap, until `resize` or other
 /// destructive method is called**.
@@ -37,9 +37,10 @@ impl<T: Element> IntoPyArray for Box<[T]> {
     type Item = T;
     type Dim = Ix1;
     fn into_pyarray<'py>(self, py: Python<'py>) -> &'py PyArray<Self::Item, Self::Dim> {
-        let len = self.len();
+        let dims = [self.len()];
         let strides = [mem::size_of::<T>() as npy_intp];
-        unsafe { PyArray::from_boxed_slice(py, [len], strides.as_ptr(), self, None) }
+        let data_ptr = self.as_ptr();
+        unsafe { PyArray::from_raw_parts(py, dims, strides.as_ptr(), data_ptr, self) }
     }
 }
 
@@ -47,7 +48,10 @@ impl<T: Element> IntoPyArray for Vec<T> {
     type Item = T;
     type Dim = Ix1;
     fn into_pyarray<'py>(self, py: Python<'py>) -> &'py PyArray<Self::Item, Self::Dim> {
-        self.into_boxed_slice().into_pyarray(py)
+        let dims = [self.len()];
+        let strides = [mem::size_of::<T>() as npy_intp];
+        let data_ptr = self.as_ptr();
+        unsafe { PyArray::from_raw_parts(py, dims, strides.as_ptr(), data_ptr, self) }
     }
 }
 
@@ -59,21 +63,7 @@ where
     type Item = A;
     type Dim = D;
     fn into_pyarray<'py>(self, py: Python<'py>) -> &'py PyArray<Self::Item, Self::Dim> {
-        let (strides, dim) = (self.npy_strides(), self.raw_dim());
-        let orig_ptr = self.as_ptr();
-        // Element of which size is 0 is not supported, but check it for future changes
-        let is_empty_or_size0 = self.is_empty() || std::mem::size_of::<A>() == 0;
-        let vec = self.into_raw_vec();
-        let offset = if is_empty_or_size0 {
-            0
-        } else {
-            (orig_ptr as usize - vec.as_ptr() as usize) / std::mem::size_of::<A>()
-        };
-        let mut boxed_slice = vec.into_boxed_slice();
-        // data_ptr is not always the pointer to the 1st element.
-        // See https://github.com/PyO3/rust-numpy/issues/182 for the detail.
-        let data_ptr = unsafe { boxed_slice.as_mut_ptr().add(offset) };
-        unsafe { PyArray::from_boxed_slice(py, dim, strides.as_ptr(), boxed_slice, Some(data_ptr)) }
+        PyArray::from_owned_array(py, self)
     }
 }
 
@@ -130,31 +120,40 @@ where
     type Dim = D;
     fn to_pyarray<'py>(&self, py: Python<'py>) -> &'py PyArray<Self::Item, Self::Dim> {
         let len = self.len();
-        if let Some(order) = self.order() {
-            // if the array is contiguous, copy it by `copy_ptr`.
-            let strides = self.npy_strides();
-            unsafe {
-                let array = PyArray::new_(py, self.raw_dim(), strides.as_ptr(), order.to_flag());
-                array.copy_ptr(self.as_ptr(), len);
-                array
+        match self.order() {
+            Some(order) if A::IS_COPY => {
+                // if the array is contiguous, copy it by `copy_ptr`.
+                let strides = self.npy_strides();
+                unsafe {
+                    let array =
+                        PyArray::new_(py, self.raw_dim(), strides.as_ptr(), order.to_flag());
+                    array.copy_ptr(self.as_ptr(), len);
+                    array
+                }
             }
-        } else {
-            // if the array is not contiguous, copy all elements by `ArrayBase::iter`.
-            let dim = self.raw_dim();
-            let strides = NpyStrides::from_dim(&dim, mem::size_of::<A>());
-            unsafe {
-                let array = PyArray::<A, _>::new_(py, dim, strides.as_ptr(), 0);
-                let data_ptr = array.data();
-                for (i, item) in self.iter().enumerate() {
-                    data_ptr.add(i).write(item.clone());
+            _ => {
+                // if the array is not contiguous, copy all elements by `ArrayBase::iter`.
+                let dim = self.raw_dim();
+                let strides = NpyStrides::new::<_, A>(
+                    dim.default_strides()
+                        .slice()
+                        .iter()
+                        .map(|&x| x as npyffi::npy_intp),
+                );
+                unsafe {
+                    let array = PyArray::<A, _>::new_(py, dim, strides.as_ptr(), 0);
+                    let data_ptr = array.data();
+                    for (i, item) in self.iter().enumerate() {
+                        data_ptr.add(i).write(item.clone());
+                    }
+                    array
                 }
-                array
             }
         }
     }
 }
 
-enum Order {
+pub(crate) enum Order {
     Standard,
     Fortran,
 }
@@ -168,7 +167,7 @@ impl Order {
     }
 }
 
-trait ArrayExt {
+pub(crate) trait ArrayExt {
     fn npy_strides(&self) -> NpyStrides;
     fn order(&self) -> Option<Order>;
 }
@@ -179,10 +178,7 @@ where
     D: Dimension,
 {
     fn npy_strides(&self) -> NpyStrides {
-        NpyStrides::new(
-            self.strides().iter().map(|&x| x as npyffi::npy_intp),
-            mem::size_of::<A>(),
-        )
+        NpyStrides::new::<_, A>(self.strides().iter().map(|&x| x as npyffi::npy_intp))
     }
 
     fn order(&self) -> Option<Order> {
@@ -196,40 +192,27 @@ where
     }
 }
 
-/// Numpy strides with short array optimization
-enum NpyStrides {
-    Short([npyffi::npy_intp; 8]),
-    Long(Vec<npyffi::npy_intp>),
-}
+/// An array of strides sufficiently large for [any NumPy array][NPY_MAXDIMS]
+///
+/// [NPY_MAXDIMS]: https://github.com/numpy/numpy/blob/4c60b3263ac50e5e72f6a909e156314fc3c9cba0/numpy/core/include/numpy/ndarraytypes.h#L40
+pub(crate) struct NpyStrides([npyffi::npy_intp; 32]);
 
 impl NpyStrides {
-    fn as_ptr(&self) -> *const npy_intp {
-        match self {
-            NpyStrides::Short(inner) => inner.as_ptr(),
-            NpyStrides::Long(inner) => inner.as_ptr(),
-        }
+    pub(crate) fn as_ptr(&self) -> *const npy_intp {
+        self.0.as_ptr()
     }
-    fn from_dim<D: Dimension>(dim: &D, type_size: usize) -> Self {
-        Self::new(
-            dim.default_strides()
-                .slice()
-                .iter()
-                .map(|&x| x as npyffi::npy_intp),
-            type_size,
-        )
-    }
-    fn new(strides: impl ExactSizeIterator<Item = npyffi::npy_intp>, type_size: usize) -> Self {
-        let len = strides.len();
-        let type_size = type_size as npyffi::npy_intp;
-        if len <= 8 {
-            let mut res = [0; 8];
-            for (i, s) in strides.enumerate() {
-                res[i] = s * type_size;
-            }
-            NpyStrides::Short(res)
-        } else {
-            NpyStrides::Long(strides.map(|n| n as npyffi::npy_intp * type_size).collect())
+
+    fn new<S, A>(strides: S) -> Self
+    where
+        S: Iterator<Item = npyffi::npy_intp>,
+    {
+        let type_size = mem::size_of::<A>() as npyffi::npy_intp;
+        let mut res = [0; 32];
+        for (i, s) in strides.enumerate() {
+            *res.get_mut(i)
+                .expect("Only dimensionalities of up to 32 are supported") = s * type_size;
         }
+        Self(res)
     }
 }
 
